/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: main.c,v 1.9 1999/05/04 08:12:03 bazsi Exp $
 *
 ***************************************************************************/

#include "syslog-ng.h"
#include "cfgfile.h"
#include "afinter.h"
#include "werror.h"

#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <getopt.h>

#include "main.c.x"

/* CLASS:
   (class
     (name reread_config)
     (super callback)
     (vars
       (cfg object syslog_conf)))
*/

static void do_reread_config(struct callback *c)
{
	CAST(reread_config, self, c);

	CONFIG_INIT(self->cfg);
	notice("new configuration initialized\n");
}

int sighuprecvd = 0;

void sig_hup(int signo)
{
	sighuprecvd = 1;
	signal(SIGHUP, sig_hup);
}

int sigtermrecvd = 0;

void sig_term(int signo)
{
	sigtermrecvd = 1;
}

int main_loop(struct syslog_conf *cfg, struct io_backend *backend)
{
	signal(SIGPIPE, SIG_IGN);
	signal(SIGHUP, sig_hup);

	while (io_iter(backend)) {
		if (sighuprecvd) {
			struct syslog_conf *newcfg;

			notice("SIGHUP received restarting syslog-ng\n");
			newcfg = make_syslog_conf(PATH_SYSLOG_NG_CONF, backend);
			if (newcfg) {
				struct reread_config r = { { STATIC_HEADER, do_reread_config }, newcfg };
				CONFIG_DESTROY(cfg);
				cfg = newcfg;
				io_callout(backend, 0, &r.super);
			}
			else {
				notice("Syntax error reading config file, using old configuration\n");
			}
			sighuprecvd = 0;
		}
	}
	return 0;
}

void go_background()
{
	pid_t pid;
	
	pid = fork();
	if (pid == 0) {
		return;
	}
	else if (pid == -1) {
		werror("Cannot fork(), (%z)\n", strerror(errno));
		exit(1);
	}
	signal(SIGTERM, sig_term);
	while (sigtermrecvd == 0)
		pause();
	exit(0);
}

void usage(void)
{
	werror("Usage: syslog-ng [options]\n"
	       "Accept and manage system log messages\n\n"
	       "Options:\n"
	       "  -d, --debug        Turn on debugging messages\n"
	       "  -v, --verbose      Enable verbosity\n");

	exit(0);
}

int main(int argc, char **argv)
{
	struct syslog_conf *cfg;
	int do_fork = 1;
	int opt;

	NEW(io_backend, backend);
	struct option syslog_ng_options[] = {
		{ "debug", no_argument, NULL, 'd' },
		{ "verbose", no_argument, NULL, 'v' },
		{ "help", no_argument, NULL, 'h' },
		{ NULL, 0, NULL, 0 }
	};

	while ((opt = getopt_long(argc, argv, "dv", syslog_ng_options, NULL)) != -1) {
		switch (opt) {
		case 'd':
			debug_flag++;
			break;
		case 'v':
			verbose_flag++;
			break;
		case 'h':
		default:
			usage();
			break;
		}
	}

	init_backend(backend);
	
	cfg = make_syslog_conf(PATH_SYSLOG_NG_CONF, backend);
	if (!cfg) {
		werror("Parse error reading configuration file\n");
		return 1;
	}
	if (debug_flag || verbose_flag)
		do_fork = 0;
	if (do_fork)
		go_background();
	if (!CONFIG_INIT(cfg)) {
		werror("Error initializing configuration\n");
		return 2;
	}

	if (do_fork) {
		set_error_internal();
		kill(getppid(), SIGTERM); 
	}
	close(0);
	close(1);
	close(2);
	setsid();
	notice("syslog-ng version %z starting\n", VERSION);
	return main_loop(cfg, backend);
}
